import java.io.*;


/* main class */
public class PM2IW_Converter {

/* ====================================================================================================================== */
	public static void main(String[] args) 
	{
		PM2IW pm2iw = new PM2IW();

		/* init process */
		displayHeader(); 
		getUserArgs(pm2iw, args);

		/* read input file (ProMotion .txm file) */
		readPMData(pm2iw);

		/* write output file (Impact Weltmeister .js file) */
		writeIWData(pm2iw);

		/* copy the tile file from ProMotion dir to Weltmeister dir */
		copyPMTilesToIWDir(pm2iw);

		System.out.println("\n-> OK!\n Output files are:\n - IWlevel: "+pm2iw.full_IWlevel+" \n - IWtiles: "+pm2iw.full_IWtileset+"\n\n");

	} /* end of 'main()' */



/* ====================================================================================================================== */
	private static void displayHeader()
	{
		System.out.println("\n\n");
		System.out.println("+===========================================+");
		System.out.println("| ProMotion2ImpactWeltmeister Converter 1.0 |");
		System.out.println("+===========================================+\n");

		System.out.println("usage:   java PM2IW_Converter 1)inputDir\\PMlevelfile.txm  2)inputDir\\PMtileset.gif/png/...  3)ImpactRootDir  4)(outputDir\\)IWlevelfile.js  5)(outputDir\\)IWtileset.gif/png/...  6)nbTilesX 7)nbTilesY 8)tileSize 9)PMemptytileID 10)IWdistance\n");
		System.out.println("example: java PM2IW_Converter C:\\promotion\\myLevel.txm    C:\\promotion\\myTiles.gif          C:\\impact        myGame\\myLevel.js             myTiles\\myTiles.gif                  30         20         16         27              3");
		System.out.println("         (note: args #4 and #5 refer to Impact's \\lib\\game\\levels\\ and \\media\\ directories respectively)\n");
	}


/* ====================================================================================================================== */
	private static void getUserArgs(PM2IW pm2iw, String[] args)
	{
		int i=0;
	
		if (args.length != PM2IW_Constants.COMMANDLINE_NBARGS)
			{ System.err.print("\n -> ERROR: wrong number of args (see help above)! Abort conversion and exit!\n\n"); System.exit(-1); }

		System.out.println("your configuration: ");
		for (String s: args) 
		{
			switch(i)
			{ 
				case(0) : 
					pm2iw.PMlevel   = args[0];	
					testIfDirOrFileExists(pm2iw.PMlevel); 	
					System.out.print("-> PMlevel   = "+pm2iw.PMlevel);			
					break;

				case(1) : 
					pm2iw.PMtileset = args[1];	
					testIfDirOrFileExists(pm2iw.PMtileset);	
					System.out.print("-> PMtileset = "+pm2iw.PMtileset);			
					break;

				case(2) : 
					pm2iw.ImpactDir = args[2];	
					pm2iw.ImpactDir=testIfDirOrFileExists(pm2iw.ImpactDir);	
					System.out.print("-> ImpactDir = "+pm2iw.ImpactDir);	
					break;

				case(3) : 
					pm2iw.IWlevel   = args[3];	
					/* test if dir exists done later (needs fullpath) */	
					System.out.print("-> IWlevel   = "+pm2iw.ImpactDir+PM2IW_Constants.IMPACT_LEVEL_DIR+pm2iw.IWlevel);		
					break;

				case(4) : 
					pm2iw.IWtileset = args[4];	
					/* test if dir exists done later (needs fullpath) */	
					System.out.print("-> IWtileset = "+pm2iw.ImpactDir+PM2IW_Constants.IMPACT_MEDIA_DIR+pm2iw.IWtileset);	
					break;

				case(5) : 
					pm2iw.nbTilesX  = Integer.parseInt(args[5]);
					System.out.print("-> nbTilesX  = "+pm2iw.nbTilesX);
					break;

				case(6) : 
					pm2iw.nbTilesY  = Integer.parseInt(args[6]);
					System.out.print("-> nbTilesY  = "+pm2iw.nbTilesY);
					break;

				case(7) : 
					pm2iw.tileSize  = Integer.parseInt(args[7]);
					System.out.print("-> tileSize  = "+pm2iw.tileSize+" (res="+pm2iw.nbTilesX*pm2iw.tileSize+"x"+pm2iw.nbTilesY*pm2iw.tileSize+")"); 
					break;

				case(8) : 
					pm2iw.PMemptyID = Integer.parseInt(args[8]);
					System.out.print("-> PMemptyID = "+pm2iw.PMemptyID);
					break;

				case(9) : 
					pm2iw.IWdistance= Integer.parseInt(args[9]);
					System.out.print("-> IWdistance= "+pm2iw.IWdistance);
					break;

				default : break;
			}

            System.out.print("\n");
            
            i++; /* next arg */
        }
		
		pm2iw.full_IWlevel  = pm2iw.ImpactDir+PM2IW_Constants.IMPACT_LEVEL_DIR+pm2iw.IWlevel;

		pm2iw.full_IWtileset= pm2iw.ImpactDir+PM2IW_Constants.IMPACT_MEDIA_DIR+pm2iw.IWtileset;
		pm2iw.IWtileset = pm2iw.IWtileset.replaceAll("\\\\", "/");
		
		{ /* output dirs exist? exit prog if not! */
			File f = new File(pm2iw.full_IWlevel);testIfDirOrFileExists(f.getParent()); 
			f = new File(pm2iw.full_IWtileset);	testIfDirOrFileExists(f.getParent());
		}
		
		/* int values are ok? */
		if ((pm2iw.nbTilesX<1) || (pm2iw.nbTilesY<1) || (pm2iw.tileSize<1) || (pm2iw.PMemptyID<0) || (pm2iw.IWdistance<1))
			{ System.err.println("\n -> ERROR: nbTilesX>0, nbTilesY>0, tileSize>0, PMemptyID>=0, and IWdistance>0! Check these values! Abort conversion and exit!\n\n"); System.exit(-2); }
	}


/* ====================================================================================================================== */
	private static void readPMData(PM2IW pm2iw)	/* read data from the .txm file (PM) and fill IW's data arrays (IWmainLayerData and IWCollisionLayerData) */
	{
		FileInputStream nomLogique = null;

		try 
			{ nomLogique = new FileInputStream(pm2iw.PMlevel); } 
		catch (FileNotFoundException e1) 
			{ e1.printStackTrace(); System.err.println ("\n -> ERROR: unable to create FileInputStream! Abort conversion and exit!\n\n"); System.exit(-3); }

		InputStreamReader ligne = new InputStreamReader (nomLogique);
		BufferedReader bufferLigne = new BufferedReader (ligne);

		try {
				int nbReadLines=0, 				/* 0 <= nbRL < nbTilesY */
					nbReadTilesInCurrentLine=0;	/* 0 <= nbRT < nbTilesX */
				
			  /* read all lines in the ProMotion file (.txm) */
			  while (true) 
			  {
				  String line = null;

				  /* read a complete line */
				  try 
				  	{ line = bufferLigne.readLine(); } 
				  catch (IOException e) 
				  	{ e.printStackTrace(); System.err.println ("\n -> ERROR: unable to read line! Abort conversion and exit!\n\n"); System.exit(-4); }

				  /* no more line to be read : input file has been fully processed -> OK! Now exit from this section (ready to write output data)... */
				  if (line == null)
					  break;

				  /* ... otherwise, parsing .txm file's  current line */
				  /* all the tiles from the current line are divided into individual tokens (tile-parsing) */
				  String[] fields = line.split(",");
				  {
					  if (fields.length != pm2iw.nbTilesX) /* error! expected number of tiles per line doesn't match actual number of tiles per line */
						  { System.err.println ("\n -> ERROR: wrong number of tiles per line specified! "+pm2iw.nbTilesX+" tiles expected on X axis, "+fields.length+" found in .txm file! Abort conversion and exit!\n\n"); System.exit(-5); }
				  }
				  
				  /* read all the tiles from the current line */
				  for (String s: fields) 
				  { 
					  if (nbReadTilesInCurrentLine == 0)		/* new line starting: no tiles of this line have been read yet (init data arrays) */
					  {
						  pm2iw.IWmainLayerData += "[";
						  pm2iw.IWcollisionLayerData += "[";
					  }

					  /* fill data arrays (IWmainLayerData and IWCollisionLayerData) */
					  if (Integer.parseInt(s) != pm2iw.PMemptyID)/* the current tile is a solid one */
					  {
						  pm2iw.IWmainLayerData += (Integer.parseInt(s)+1);
						  pm2iw.IWcollisionLayerData += PM2IW_Constants.IMPACT_LEVEL_SOLID_TILE;
					  }
					  else										/* empty tile */
					  {
						  pm2iw.IWmainLayerData += PM2IW_Constants.IMPACT_LEVEL_EMPTY_TILE;
						  pm2iw.IWcollisionLayerData += PM2IW_Constants.IMPACT_LEVEL_EMPTY_TILE;
					  }
					  
					  nbReadTilesInCurrentLine++;				/* ok: job done for the current tile ; now next ... */


					  if (nbReadTilesInCurrentLine < pm2iw.nbTilesX)	/* test if we reach the last tile of the current line */
					  { 
						  pm2iw.IWmainLayerData += ",";			/* add a ',' between each read tile (but not after the last tile of the line) */
						  pm2iw.IWcollisionLayerData += ",";	/* add a ',' between each read tile (but not after the last tile of the line) */
					  }
					  else // if (nbReadTilesInCurrentLine == nbTilesX)	/* all the tiles from the current line have been read : new line about to be read */
					  { 
						  pm2iw.IWmainLayerData += "]";			/* mark the end of the current line of data */
						  pm2iw.IWcollisionLayerData += "]";	/* mark the end of the current line of data */
						  
						  nbReadLines++;						/* new line */
						  nbReadTilesInCurrentLine=0;			/* new line: reset tile counter */
						  
						  if (nbReadLines < pm2iw.nbTilesY)		/* test if we reach the last line of .txm file */
						  { 
							  pm2iw.IWmainLayerData += ",";		/* add a ',' between each line of tiles (but not after the last line of tiles) */
							  pm2iw.IWcollisionLayerData += ",";/* add a ',' between each line of tiles (but not after the last line of tiles) */
						  }			  
					  }						  
				  } /* end of 'for(read all the tiles in the current line)' */
			  }		/* end of 'while(read all the lines in .txm file)' */
			  
			  if (nbReadLines != pm2iw.nbTilesY)		/* error! expected number of lines to read doesn't match actual number of lines in .txm file */
			  	{ System.err.println ("\n -> ERROR: wrong number of lines specified! "+pm2iw.nbTilesY+" tiles expected on Y axis, "+nbReadLines+" found in .txm file! Abort conversion and exit!\n\n"); System.exit(-6); }
		} 
		finally 
		{
			try 
				{ bufferLigne.close(); } 
			catch (IOException e) 
				{ e.printStackTrace(); System.err.println ("\n -> ERROR: unable to close input file! Abort conversion and exit!\n\n"); System.exit(-7); }
		}
	}


/* ====================================================================================================================== */
	private static void writeIWData(PM2IW pm2iw) /* write the .js file (IW) thanks to IW's data arrays (IWmainLayerData and IWCollisionLayerData) */ 
	{
		FileOutputStream fout;	// stream to write file	

		try
		{
			String short_IWlevelname = null; // just the level name, no dir, and no file extension (.js)
	
		    fout = new FileOutputStream (pm2iw.full_IWlevel);	// open an output stream

	    	{ String[] short_IWlevel = pm2iw.IWlevel.split("\\."); pm2iw.IWlevel = short_IWlevel[0]; }	// in the ouptput file, for the 'ig.module' line, ... 
	    	pm2iw.IWlevel = pm2iw.IWlevel.replaceAll("\\\\", "\\.");		// remove the file extension (.js)(line above) and  turn all optional "\" into "." (this line)

		    new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_MODULE_BEGIN+pm2iw.IWlevel+PM2IW_Constants.IMPACT_LEVEL_MODULE_END);
		    
		    new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_REQUIRES);
		    new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_DEFINES_BEGIN);
		    
		    	/* the file extension (.js) has previously been removed, now get rid of the optional dir(s) */
		    	if (pm2iw.IWlevel.contains(".")) 
		    		{ short_IWlevelname = pm2iw.IWlevel.substring(pm2iw.IWlevel.lastIndexOf(".")+1); } /* remove subdirs in order to name the level */
		    	else 
		    		{ short_IWlevelname = pm2iw.IWlevel; } /* no subdir: we already know the name of the level */
		    	
		    	/* capitalize 1st letter */
		    	short_IWlevelname = Character.toUpperCase(short_IWlevelname.charAt(0))+short_IWlevelname.substring(1);
		    	new PrintStream(fout).println ("Level"+short_IWlevelname+"=");

		    	new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_JSON_BEGIN);
		    	
		    		new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_BEGIN);

		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_COLLISION_01+pm2iw.nbTilesX);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_COLLISION_02+pm2iw.nbTilesY);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_COLLISION_03+pm2iw.tileSize);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_COLLISION_04);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_COLLISION_05+pm2iw.IWcollisionLayerData);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_COLLISION_06);
		    			
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_01+short_IWlevelname+PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_02+pm2iw.nbTilesX);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_03+pm2iw.nbTilesY);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_04+pm2iw.IWtileset+PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_05+pm2iw.IWdistance);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_06+pm2iw.tileSize);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_07);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_08+pm2iw.IWmainLayerData);
		    			new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_PROMOTION_09);
		    
		    		new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_LAYER_END);
		    
		    	new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_JSON_END);
		    	
		    	new PrintStream(fout).println ("Level"+short_IWlevelname+PM2IW_Constants.IMPACT_LEVEL_RESOURCES_BEGIN+pm2iw.IWtileset+PM2IW_Constants.IMPACT_LEVEL_RESOURCES_END);
		    	
		    new PrintStream(fout).println (PM2IW_Constants.IMPACT_LEVEL_DEFINES_END);
		    
		    fout.close(); // close our output stream		
		}
		catch (IOException e)
			{ e.printStackTrace(); System.err.println ("\n -> ERROR: unable to write output file! Abort conversion and exit!\n\n"); System.exit(-8); }

	}


/* ====================================================================================================================== */	
	private static void copyPMTilesToIWDir(PM2IW pm2iw)
	{
		FileInputStream fis = null;
		try { fis = new FileInputStream(pm2iw.PMtileset); }
		catch (FileNotFoundException e1) { e1.printStackTrace(); }
		
		FileOutputStream fos= null;
		try { fos = new FileOutputStream(pm2iw.full_IWtileset); }
		catch (FileNotFoundException e1) { e1.printStackTrace(); }
		
		File ficIn = new File(pm2iw.PMtileset);					/* read data from PM's tile file */
		byte bytesTileset[] = new byte[(int) ficIn.length()];
		try 
		{
			while(fis.read(bytesTileset) != -1)
				{;}
		} 
		catch (IOException e) { e.printStackTrace(); }
		
		try { fos.write(bytesTileset); } 						/* write IW's tile file */
		catch (IOException e) { e.printStackTrace(); }

		
		try { fis.close(); } 
		catch (IOException e) { e.printStackTrace(); }
		
		try { fos.close(); } 
		catch (IOException e) { e.printStackTrace(); }
	}


/* ====================================================================================================================== */
	private static String testIfDirOrFileExists(String path)
	{
		File testFile = new File(path);
		if (!testFile.exists())
			{ System.err.println("\n -> ERROR: file or directory '"+path+"' doesn't exist. Check (and create?) the specified path! Abort conversion and exit!\n\n"); System.exit(-10); }

		/* for dirs, remove final '\' if any */
		if(path.charAt(path.length()-1)=='\\')	
			{ path = path.substring(0, path.length()-1); }

		return(path);
	}

} /* end of class */


